# -*- coding: utf-8 -*-
"""
API status endpoints
"""
from __future__ import absolute_import, unicode_literals

from django.conf.urls import url

from ..apps import FonzieConfig
from ..views.status import VersionView

app_name = FonzieConfig.name
urlpatterns = [url(r"^version$", VersionView.as_view(), name="version")]



# -*- coding: utf-8 -*-
"""
Access control to instructor files API endpoint
"""
from __future__ import absolute_import, unicode_literals

from django.conf.urls import url

from ..apps import FonzieConfig
from ..views.acl import ReportView

app_name = FonzieConfig.name
urlpatterns = [
    url(
        r"^report/(?P<course_sha1>[a-f0-9]{40})/(?P<filename>[\d\w\-\_\.\+\@]+)$",
        ReportView.as_view(),
        name="report",
    )
]



"""
Instructor file API endpoint
"""
# pylint: disable=import-error
from __future__ import absolute_import, unicode_literals

import hashlib

import six
from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from student.models import CourseAccessRole
from student.roles import CourseStaffRole


class ReportView(APIView):
    """API endpoint to control access to CSV export files"""

    permission_classes = (IsAuthenticated,)

    # pylint: disable=unused-argument
    def get(self, request, version, course_sha1, filename):
        """
        API endpoint that control access to instructors CSV export files,
        then redirects to Nginx.
        Users requesting access to a CSV file sould be superuser or
        belong to course staff.
        """

        # Retrieve all courses for which user is staff and compare their sha1 to
        # the requested one.
        if not request.user.is_superuser:
            courses = CourseAccessRole.objects.filter(
                user=request.user, role=CourseStaffRole.ROLE
            )
            for course in courses:
                hashed_course_id = hashlib.sha1(  # nosec
                    six.text_type(course.course_id)
                ).hexdigest()
                if hashed_course_id == course_sha1:
                    break
            else:
                # None matches, user does not have rights to request this file
                return Response({}, status=status.HTTP_403_FORBIDDEN)

        # Redirect browser to an internal Nginx location
        response = Response({})
        response["Content-Disposition"] = "attachment; filename={0}".format(filename)
        response["X-Accel-Redirect"] = "/restricted/{course_sha1}/{filename}".format(
            course_sha1=course_sha1, filename=filename
        )
        return response



# coding: utf-8
"""
Tests for the `fonzie` user module.
"""

# pylint: disable=no-member,import-error
from __future__ import absolute_import, unicode_literals

import random

import jwt
from rest_framework import status
from rest_framework.test import APITestCase

from django.core.urlresolvers import reverse
from django.test import override_settings

from openedx.core.djangoapps.user_api.tests.factories import UserPreferenceFactory
from student.tests.factories import UserFactory, UserProfileFactory


class UserViewTestCase(APITestCase):
    """Tests for the User API endpoint"""

    def setUp(self):
        """
        Set common parameters for the test suite.
        """
        super(UserViewTestCase, self).setUp()

        self.url = reverse("fonzie:user:me", kwargs={"version": "1.0"})

    def test_user_me_with_anonymous_user(self):
        """
        If user is not authenticated, view should return a 403 status
        """
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @override_settings(
        LANGUAGE_CODE="de",
        FEATURES={'ENABLE_CROSS_DOMAIN_CSRF_COOKIE': True},
        CROSS_DOMAIN_CSRF_COOKIE_NAME="edx_csrf_token",
        CROSS_DOMAIN_CSRF_COOKIE_DOMAIN="localhost",
    )
    def test_user_me_with_logged_in_user(self):
        """
        If user is authenticated through Django session, view should return
        a JSON object containing the username and a JWT access token and a cross domain
        csrf token cookie should be set.
        """
        user = UserFactory.create(
            username="fonzie",
            email="arthur_fonzarelli@fun-mooc.fr",
            is_active=random.choice([True, False]),
            is_staff=random.choice([True, False]),
            is_superuser=random.choice([True, False]),
        )
        UserProfileFactory.build(user=user, name="Arthur Fonzarelli")
        self.client.force_authenticate(user=user)

        response = self.client.get(self.url)
        token = jwt.decode(
            response.data["access_token"],
            "ThisIsAnExampleKeyForDevPurposeOnly",
            options={
                "require": [
                    "email",
                    "exp",
                    "iat",
                    "jti",
                    "token_type",
                    "username",
                    "language",
                    "is_active",
                    "is_staff",
                    "is_superuser",
                ]
            },
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.csrf_cookie_set, True)
        self.assertEqual(response.cookies.get('edx_csrf_token').key, 'edx_csrf_token')
        self.assertEqual(response.cookies.get('edx_csrf_token').get('domain'), 'localhost')
        self.assertEqual(response.data["username"], "fonzie")
        self.assertEqual(token["username"], "fonzie")
        self.assertEqual(token["full_name"], "Arthur Fonzarelli")
        self.assertEqual(token["email"], "arthur_fonzarelli@fun-mooc.fr")
        self.assertEqual(token["is_active"], user.is_active)
        self.assertEqual(token["is_staff"], user.is_staff)
        self.assertEqual(token["is_superuser"], user.is_superuser)
        # When the user has no language preference, LANGUAGE_CODE should be used
        self.assertEqual(token["language"], 'de')
        self.assertEqual(token["token_type"], "access")
        self.assertIsInstance(token["exp"], int)
        self.assertIsInstance(token["iat"], int)

    def test_user_me_with_logged_in_user_and_language_preference(self):
        """
        If user is authenticated through Django session, and it sets a language
        in its preferences, the JWT access token returned by the view should contain
        the language.
        """
        user = UserFactory.create(
            username="joanie",
            email="joanie_cunningham@fun-mooc.fr",
        )
        UserPreferenceFactory(user=user, key="pref-lang", value="fr")
        self.client.force_authenticate(user=user)

        response = self.client.get(self.url)
        token = jwt.decode(
            response.data["access_token"],
            "ThisIsAnExampleKeyForDevPurposeOnly",
            options={
                "require": [
                    "email",
                    "exp",
                    "iat",
                    "jti",
                    "token_type",
                    "username",
                    "language",
                    "is_active",
                    "is_staff",
                    "is_superuser",
                ]
            },
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data["username"], "joanie")
        self.assertEqual(token["email"], "joanie_cunningham@fun-mooc.fr")
        self.assertEqual(token["username"], "joanie")
        self.assertEqual(token["language"], "fr")



# -*- coding: utf-8 -*-
"""
Tests for the `fonzie` acl module.
"""
# pylint: disable=no-member,import-error
from __future__ import absolute_import, unicode_literals

import hashlib

import six
from rest_framework import status
from rest_framework.test import APITestCase

from django.core.urlresolvers import reverse

from student.roles import CourseStaffRole
from student.tests.factories import UserFactory
from xmodule.modulestore import ModuleStoreEnum
from xmodule.modulestore.tests.django_utils import ModuleStoreTestCase
from xmodule.modulestore.tests.factories import CourseFactory


class ReportViewTestCase(ModuleStoreTestCase, APITestCase):
    """Tests for the InstructorFilesView API endpoint"""

    def setUp(self):
        """
        Set common parameters for unit tests.
        """
        super(ReportViewTestCase, self).setUp()

        # this file format is used for student profiles
        self.filename = "fun_test_00_student_profile_info_2019-04-19-1406.csv"
        self.course = CourseFactory.create(default_store=ModuleStoreEnum.Type.split)
        self.course_staff = UserFactory(
            username="course_staff", email="course_staff@fun-mooc.fr"
        )
        CourseStaffRole(self.course.id).add_users(self.course_staff)
        self.course_sha1 = hashlib.sha1(six.text_type(self.course.id)).hexdigest()

        self.url = reverse(
            "fonzie:acl:report",
            kwargs={
                "version": "1.0",
                "course_sha1": self.course_sha1,
                "filename": self.filename,
            },
        )

    def test_user_is_not_logged_in(self):
        """
        User should be logged in, if not the view should return 403
        """
        response = self.client.get(self.url, format="json")
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_user_is_not_staff(self):
        """
        User should not be a just student, if so the view should return 403
        """
        normal_user = UserFactory(
            username="normal_user", email="normal_user@fun-mooc.fr"
        )
        self.client.force_authenticate(normal_user)
        response = self.client.get(self.url, format="json")
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_user_is_staff_on_other_course(self):
        """
        User should belong to this course staff, if not the view should return 403
        """
        course = CourseFactory.create(default_store=ModuleStoreEnum.Type.split)
        other_course_staff = UserFactory(
            username="other_course_staff", email="other_course_staff@fun-mooc.fr"
        )
        CourseStaffRole(course.id).add_users(other_course_staff)
        self.client.force_authenticate(other_course_staff)
        response = self.client.get(self.url, format="json")
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_user_has_correct_rights(self):
        """
        User fullfils all conditions, then he should be redirected to Nginx
        """
        self.client.force_authenticate(self.course_staff)
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.get("X-Accel-Redirect"),
            "/restricted/%s/%s" % (self.course_sha1, self.filename),
        )
        self.assertEqual(
            response.get("Content-Disposition"),
            "attachment; filename=%s" % self.filename,
        )

    def test_user_can_download_problem_response(self):
        """
        User fullfils all conditions and downloads a problem response file,
        he should be redirected to Nginx
        """
        # this file format is used by problem responses files
        filename = (
            "edX_DemoX_Demo_Course_student_state_from_block-v1_edX+DemoX"
            "+Demo_Course+type@problem+block@d1b84dcd39b0423d9e288f27f0f7f242_"
            "2019-10-09-1219.csv"
        )
        url = reverse(
            "fonzie:acl:report",
            kwargs={
                "version": "1.0",
                "course_sha1": self.course_sha1,
                "filename": filename,
            },
        )

        self.client.force_authenticate(self.course_staff)
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.get("X-Accel-Redirect"),
            "/restricted/%s/%s" % (self.course_sha1, filename),
        )
        self.assertEqual(
            response.get("Content-Disposition"),
            "attachment; filename=%s" % filename,
        )

    def test_user_has_correct_rights_old_course_id_format(self):
        """
        Ensure endpoint also works with legacy course id format
        (org.1/course_1/Run_1 against course-v1:org.1+course_1+Run_1)
        """
        course = CourseFactory.create()  # create a course with old course_id format
        CourseStaffRole(course.id).add_users(self.course_staff)
        course_id = six.text_type(course.id)
        self.client.force_authenticate(self.course_staff)
        url = reverse(
            "fonzie:acl:report",
            kwargs={
                "version": "1.0",
                "course_sha1": hashlib.sha1(course_id).hexdigest(),
                "filename": self.filename,
            },
        )
        response = self.client.get(url)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.get("X-Accel-Redirect"),
            "/restricted/%s/%s"
            % (hashlib.sha1(six.text_type(course_id)).hexdigest(), self.filename),
        )

    def test_user_is_superuser(self):
        """
        Super users should always get requested file.
        """
        super_user = UserFactory(
            username="super_user",
            email="super_user@fun-mooc.fr",
            is_superuser=True
        )
        self.client.force_authenticate(super_user)
        response = self.client.get(self.url, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.get("X-Accel-Redirect"),
            "/restricted/%s/%s" % (self.course_sha1, self.filename),
        )
        self.assertEqual(
            response.get("Content-Disposition"),
            "attachment; filename=%s" % self.filename,
        )



# -*- coding: utf-8 -*-
"""
Tests for the `fonzie` views module.
"""

from __future__ import absolute_import, unicode_literals

from rest_framework import status  # pylint: disable=import-error
from rest_framework.test import APITestCase  # pylint: disable=import-error

from django.core.urlresolvers import reverse

from fonzie import __version__ as fonzie_version


class VersioningSchemaTestCase(APITestCase):
    """Tests for the API versioning schema"""

    def setUp(self):
        """Set view url"""
        super(VersioningSchemaTestCase, self).setUp()

        self.url = reverse("fonzie:status:version", kwargs={"version": "1.0"})

    def test_url_path_versioning(self):
        """Test allowed API versions in URL Path API versioning"""

        # Default API version: 1.0
        url = reverse("fonzie:status:version", kwargs={"version": "1.0"})
        self.assertIn("v1.0", url)
        response = self.client.get(url, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertJSONEqual(response.content, {"version": fonzie_version})

        # Unimplemented or forbidden API version
        url = reverse("fonzie:status:version", kwargs={"version": "1.1"})
        self.assertIn("v1.1", url)
        response = self.client.get(url, format="json")
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertJSONEqual(
            response.content, {"detail": "Invalid version in URL path."}
        )



# -*- coding: utf-8 -*-
"""
fonzie Django application initialization.
"""

from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig


class FonzieConfig(AppConfig):
    """Configuration for the fonzie Django application."""

    name = "fonzie"



# -*- coding: utf-8 -*-
"""
API status views
"""
from __future__ import absolute_import, unicode_literals

from rest_framework.response import Response  # pylint: disable=import-error
from rest_framework.views import APIView  # pylint: disable=import-error

from fonzie import __version__ as fonzie_version


class VersionView(APIView):
    """API endpoint to get the running API version"""

    # pylint: disable=redefined-builtin
    def get(self, request, version, format=None):
        """Retrieve API version as a SemVer string"""
        return Response({"version": fonzie_version})



# -*- coding: utf-8 -*-
"""
URLs for fonzie.
"""
from __future__ import absolute_import, unicode_literals

from django.conf.urls import include, url

from ..apps import FonzieConfig
from . import acl as acl_urls
from . import status as status_urls
from . import user as user_urls

app_name = FonzieConfig.name

# For now, as DRF OpenAPI only supports URLPathVersioning, we prefix API root
# URL by the API release number. *This is a temporary solution*.
# See: https://github.com/limdauto/drf_openapi#4-constraints
#
# TODO: switch to AcceptHeaderVersioning
# http://www.django-rest-framework.org/api-guide/versioning/#acceptheaderversioning
API_PREFIX = r"^v(?P<version>[0-9]+\.[0-9]+)"

urlpatterns = [
    url(r"{}/status/".format(API_PREFIX), include(status_urls, namespace="status")),
    url(
        r"{}/acl/".format(API_PREFIX),
        include(acl_urls, namespace="acl"),
    ),
    url(r"{}/user/".format(API_PREFIX), include(user_urls, namespace="user")),
]



# -*- coding: utf-8 -*-
"""
Tests for the `fonzie` views module.
"""

from __future__ import absolute_import, unicode_literals

from rest_framework import status  # pylint: disable=import-error
from rest_framework.test import APITestCase  # pylint: disable=import-error

from django.core.urlresolvers import reverse

from fonzie import __version__ as fonzie_version


class VersionViewTestCase(APITestCase):
    """Tests for the Versionview"""

    def setUp(self):
        """Set view url"""
        super(VersionViewTestCase, self).setUp()

        self.url = reverse("fonzie:status:version", kwargs={"version": "1.0"})

    def test_get(self):
        """HTTP/GET returns API version"""

        response = self.client.get(self.url, format="json")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertJSONEqual(response.content, {"version": fonzie_version})

    def test_post(self):
        """HTTP/POST should return a 405"""

        response = self.client.post(self.url, data={}, format="json")

        self.assertEqual(response.status_code, status.HTTP_405_METHOD_NOT_ALLOWED)



"""
Fonzie, A FUN API for Open edX.
"""
from __future__ import absolute_import, unicode_literals

from os import path

import pkg_resources
from setuptools.config import read_configuration


def _extract_version(package_name):
    """
    Extract fonzie version.

    Get package version from installed distribution or configuration file if not installed
    """
    try:
        return pkg_resources.get_distribution(package_name).version
    except pkg_resources.DistributionNotFound:
        _conf = read_configuration(
            path.join(path.dirname(path.dirname(__file__)), "setup.cfg")
        )
    return _conf["metadata"]["version"]


__version__ = _extract_version("fonzie")

default_app_config = "fonzie.apps.FonzieConfig"  # pylint: disable=invalid-name



# coding: utf-8
"""
API user views
"""
# pylint: disable=import-error
from __future__ import absolute_import, unicode_literals

from datetime import datetime

from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework_simplejwt.tokens import AccessToken

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.utils.decorators import method_decorator

from openedx.core.lib.api.authentication import SessionAuthenticationAllowInactiveUser

# Since Hawthorn version, the module `cors_csrf` has been moved from `common.djangoapps`
# to `openedx.core.djangoapps`. So according to the version of Open edX, we need to
# import the module from the right place.
try:
    # First try to import the module from `openedx.core.djangoapps`
    from openedx.core.djangoapps.cors_csrf.decorators import ensure_csrf_cookie_cross_domain
except ImportError:
    # Otherwise, we are using an older version of Open edX, so we import the module
    # from `common.djangoapps`
    from cors_csrf.decorators import ensure_csrf_cookie_cross_domain


class UserSessionView(APIView):
    """API endpoint to get the authenticated user information."""

    authentication_classes = [SessionAuthenticationAllowInactiveUser]
    permission_classes = [IsAuthenticated]

    # To be able to update user profile through route api/user/v1/accounts/:username
    # we need to provide a valid CSRF Token. This is why we need to ensure that
    # the CSRF cookie is set for cross domain requests.
    @method_decorator(ensure_csrf_cookie_cross_domain)
    # pylint: disable=redefined-builtin
    def get(self, request, version, format=None):
        """
        Retrieve logged in user, then generate a JWT with a claim containing its
        username (unique identifier) and its email. The token's expiration can
        be changed through the setting `ACCESS_TOKEN_LIFETIME` (default 5 minutes).
        """
        user = request.user
        issued_at = datetime.utcnow()
        try:
            language = user.preferences.get(key="pref-lang").value
        except ObjectDoesNotExist:
            language = settings.LANGUAGE_CODE
        token = AccessToken()
        token.payload.update(
            {
                "email": user.email,
                "full_name": user.profile.name,
                "iat": issued_at,
                "language": language,
                "username": user.username,
                "is_active": user.is_active,
                "is_staff": user.is_staff,
                "is_superuser": user.is_superuser,
            },
        )
        response = Response(
            {
                "access_token": str(token),
                "username": user.username,
                "full_name": user.profile.name,
            }
        )
        if "newsletter_subscribed" in request.COOKIES:
            # delete the cookie, as we only need it once, after registration
            token.payload["has_subscribed_to_commercial_newsletter"] = (
                request.COOKIES.get("newsletter_subscribed") == "true"
            )
            response.delete_cookie("newsletter_subscribed")
            response.data["access_token"] = str(token)

        return response



"""
URLs for fonzie
This file appends Fonzie urls to edX lms urls.
It is intended to be defined in lms settings this way:

`ROOT_URLCONF = "fonzie.urls.lms_root"`
"""
from __future__ import absolute_import, unicode_literals

from django.conf.urls import include, url

from lms.urls import urlpatterns  # pylint: disable=import-error

# Fonzie urls
urlpatterns += [
    url(r"^api/", include("fonzie.urls", namespace="fonzie")),
]



# coding: utf-8
"""
API user endpoints
"""
from __future__ import absolute_import, unicode_literals

from django.conf.urls import url

from ..apps import FonzieConfig
from ..views.user import UserSessionView

app_name = FonzieConfig.name
urlpatterns = [url(r"^me/?$", UserSessionView.as_view(), name="me")]
